/*
 *  Copyright (C) 2003  Jens Kanschik,
 * 	mail : jensKanschik@users.sourceforge.net
 *
 *  Part of <hypergraph>, an open source project at sourceforge.net
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package hypergraph.test;

import java.applet.AppletContext;
import java.net.URL;
import java.util.Iterator;

import javax.swing.JApplet;
import javax.swing.UIManager;

import hypergraph.graphApi.AttributeManager;
import hypergraph.graphApi.Graph;
import hypergraph.graphApi.GraphSystem;
import hypergraph.graphApi.GraphSystemFactory;
import hypergraph.graphApi.Node;
import hypergraph.graphApi.algorithms.GraphUtilities;
import hypergraph.graphApi.io.ContentHandlerFactory;
import hypergraph.graphApi.io.SAXReader;
import hypergraph.visualnet.GraphPanel;
import hypergraph.visualnet.GraphSelectionEvent;
import hypergraph.visualnet.GraphSelectionListener;
import hypergraph.visualnet.GraphSelectionModel;

public class NetApplet extends JApplet implements GraphSelectionListener {

	static Graph graph;

	public void valueChanged(GraphSelectionEvent e) {
		GraphSelectionModel gsm = (GraphSelectionModel) e.getSource();
		Iterator i = gsm.getSelectionElementIterator();
		if (i.hasNext()) {
			Node node = (Node) i.next();
			AttributeManager attrMgr = graph.getAttributeManager();
			String href = (String) attrMgr.getAttribute("xlink:href", node);
			AppletContext context = getAppletContext();
			try {
				URL url = new URL(href);
				context.showDocument(url);
			} catch (Exception ex) {
			}
		}
	}

     public void init() {
    	String file = getParameter("file");
		UIManager.put("ModelPanelUI", "hypergraph.hyperbolic.ModelPanelUI");
//	UIManager.put("ModelPanelUI","hyperbolic.HalfPlanePanelUI");
		GraphSystem graphSystem = null;
		try {
			graphSystem = GraphSystemFactory.createGraphSystem("hypergraph.graph.GraphSystemImpl", null);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(8);
		}
		graph = null;
		URL url = null;
		try {
			url = new URL(getCodeBase(), file);
			SAXReader reader = new SAXReader(graphSystem, url);
			reader.setContentHandlerFactory(new ContentHandlerFactory());
			graph = reader.parse();
		} catch (Exception e) {
			System.out.println("Exception : " + e);
			e.printStackTrace(System.out);
		}

		if (graph == null)
			graph = GraphUtilities.createTree(graphSystem, 3, 3);

    	GraphPanel graphPanel = new GraphPanel(graph);
		graphPanel.getSelectionModel().addSelectionEventListener(this);
   		getContentPane().add(graphPanel);

    }
}
